/* This test driver program encodes a data record and writes the encoded 
   result to an output file.. */

#include <stdio.h>
#include <stdlib.h>
#include "RANAP-Constants.h"
#include "RANAP-Containers.h"
#include "RANAP-IEs.h"
#include "RANAP-PDU-Contents.h"
#include "RANAP-PDU-Descriptions.h"
#include "rtxsrc/rtxMemLeakCheck.h"
#include "rtxsrc/rtxDiag.h"
#include "rtxsrc/rtxFile.h"

#define MAXMSGLEN 1024

int main (int argc, char** argv)
{
   RelocationRequest relocResAlloc;
   InitiatingMessage initMsg;
   RANAP_PDU     pduData;
   const OSOCTET* msgptr;
   OSOCTET      msgbuf[MAXMSGLEN];
   OSBOOL       verbose = FALSE, trace = TRUE, aligned = TRUE;
   OSCTXT       ctxt;
   const char*  filename = "message.dat";
   int		i, len, stat;

   DECLARE_MEMLEAK_DETECTOR;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = TRUE;
         else if (!strcmp (argv[i], "-u")) aligned = FALSE;
         else if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer -a | -u [ -v ] [ -o <filename>\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Initialize context structure */

   if (rtInitContext (&ctxt) != 0) {
      rtxErrPrint (&ctxt);
      return -1;
   }
   rtxSetDiag (&ctxt, verbose);
   pu_setBuffer (&ctxt, msgbuf, sizeof(msgbuf), aligned);
   pu_setTrace (&ctxt, trace);

   /* Populate RelocationRequest structure */

   asn1Init_RelocationRequest (&relocResAlloc);
   {
      PermanentNAS_UE_ID* pvalue = rtxMemAllocType (&ctxt, PermanentNAS_UE_ID);
      if (0 == pvalue) return RTERR_NOMEM;

      pvalue->t = T_PermanentNAS_UE_ID_iMSI;
      pvalue->u.iMSI = rtxMemAllocType (&ctxt, IMSI);
      if (0 == pvalue) return RTERR_NOMEM;
      pvalue->u.iMSI->numocts = 8;
      for (i = 0; i < 8; i++) {
         pvalue->u.iMSI->data[i] = (OSOCTET) i;
      }
      stat = asn1Append_RelocationRequestIEs_id_PermanentNAS_UE_ID
         (&ctxt, &relocResAlloc.protocolIEs, pvalue);
      if (0 != stat) return stat;
   }
   {
      Cause* pvalue = rtxMemAllocType (&ctxt, Cause);
      if (0 == pvalue) return RTERR_NOMEM;

      pvalue->t = T_Cause_misc;
      pvalue->u.misc = CauseMisc_no_resource_available;

      stat = asn1Append_RelocationRequestIEs_id_Cause
         (&ctxt, &relocResAlloc.protocolIEs, pvalue);
      if (0 != stat) return stat;
   }

   stat = asn1Append_RelocationRequestIEs_id_CN_DomainIndicator
      (&ctxt, &relocResAlloc.protocolIEs, CN_DomainIndicator_ps_domain);
   if (0 != stat) return stat;

   /* Add extension elements */

   relocResAlloc.m.protocolExtensionsPresent = 1;
   {
      GlobalCN_ID* pvalue = rtxMemAllocType (&ctxt, GlobalCN_ID);
      if (0 == pvalue) return RTERR_NOMEM;

      for (i = 0; i < 3; i++) {
         pvalue->pLMNidentity.data[i] = (OSOCTET) i;
      }
      pvalue->cN_ID = 999;

      stat = asn1Append_RelocationRequestExtensions_id_GlobalCN_ID
         (&ctxt, &relocResAlloc.protocolExtensions, pvalue);
      if (0 != stat) return stat;
   }

   /* Populate InitiatingMessage structure */

   asn1Init_InitiatingMessage (&initMsg);
   initMsg.procedureCode = ASN1V_id_RelocationResourceAllocation;
   initMsg.criticality = Criticality_reject;
   initMsg.value.t = T_RANAPPDUDescr_RANAP_EP_relocationResourceAllocation;
   initMsg.value.u.relocationResourceAllocation = &relocResAlloc;

   /* Populate RANAP-PDU structure */

   pduData.t = T_RANAP_PDU_initiatingMessage;
   pduData.u.initiatingMessage = &initMsg;

   /* Encode the entire message */

   if ((stat = asn1PE_RANAP_PDU (&ctxt, &pduData)) == 0) {
      if (trace) {
         printf ("\n******\n");
         printf ("RANAP PDU encoding was successful\n");
         printf ("Hex dump of encoded record:\n");
         pu_hexdump (&ctxt);
         printf ("Binary dump:\n");
         pu_bindump (&ctxt, "userInfo");
      }
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      return -1;
   }

   msgptr = pe_GetMsgPtr (&ctxt, &len);

   /* Write the encoded message out to the output file */

   stat = rtxFileWriteBinary (filename, msgptr, len);
   if (0 != stat) {
      rtxErrPrint (&ctxt);
      return -1;
   }

   rtxFreeContext (&ctxt);

   return 0;
}
